/* -*- c++ -*-
 *
 * Copyright (c) 2007 Anthony Mallet                  --  Sat Feb 17 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_V3D_INLINE
#define H_V3D_INLINE

/* --- V3D::V3D ------------------------------------------------------ */

/** \brief Vector constructor.
 *
 * Initialize a vector to all 0.
 */

inline
V3D::V3D(void)
{
  v3dInit(this);
}


/** \brief Vector constructor.
 *
 * Initialize a vector to x,y and z.
 * \param[in] x		X coordinate.
 * \param[in] y		Y coordinate.
 * \param[in] z		Z coordinate.
 */

inline
V3D::V3D(double x, double y, double z)
{
  v3dInit(this);
  v[0] = x;
  v[1] = y;
  v[2] = z;
}

/* -- Unary operator- ------------------------------------------------- */

/** \brief the opposite vector.
 *
 * \return the opposite vector.
 */

inline V3D 
V3D::operator-() const
{
  V3D outV (-v[0], -v[1], -v[2]);
  return outV;
};


/* -- V3D::operator+ ------------------------------------------------- */

/** \brief Sum of two vectors or of a vector and a point.
 *
 * \param[in] inV	Vector to sum.
 * \return the result of the addition.
 */

inline V3D
V3D::operator+(const V3D &inV) const
{
  V3D outV;

  outV.v[0] = v[0] + inV.v[0];
  outV.v[1] = v[1] + inV.v[1];
  outV.v[2] = v[2] + inV.v[2];
  outV.v[3] = 1;
  return outV;
}


/* -- V3D::operator+= ------------------------------------------------- */

/** \brief Sum of two vectors or of a vector and a point.
 *
 * \param[in] inV	Vector to sum.
 * \return the result of the addition.
 */

inline V3D
V3D::operator+=(const V3D &inV)
{
  v[0] += inV.v[0];
  v[1] += inV.v[1];
  v[2] += inV.v[2];
  v[3] = 1;

  return *this;
}

/* -- V3D::operator- ------------------------------------------------- */

/** \brief Substraction of two vectors or of a vector and a point.
 *
 * \param[in] inV	Vector to substract.
 * \return the result of the substraction.
 */

inline V3D
V3D::operator-(const V3D &inV) const
{
  V3D outV;

  outV.v[0] = v[0] - inV.v[0];
  outV.v[1] = v[1] - inV.v[1];
  outV.v[2] = v[2] - inV.v[2];
  outV.v[3] = 1;
  return outV;
}

/* -- V3D::operator-= ------------------------------------------------- */

/** \brief Substraction of two vectors or of a vector and a point.
 *
 * \param[in] inV	Vector to substract.
 * \return the result of the substraction.
 */

inline V3D
V3D::operator-=(const V3D &inV)
{
  v[0] -= inV.v[0];
  v[1] -= inV.v[1];
  v[2] -= inV.v[2];
  v[3] = 1;

  return *this;
}

/* -- V3D::operator/ ------------------------------------------------- */

/** \brief Divide the vector by an double.
 *
 * \param[in] d	double to divide
 * \return the result of the division.
 */

inline V3D
V3D::operator/(const double d) const
{
  V3D outV;

  outV.v[0] = v[0]/d;
  outV.v[1] = v[1]/d;
  outV.v[2] = v[2]/d;
  outV.v[3] = 1;
  return outV;
}


/* -- V3D::operator/= ------------------------------------------------- */

/** \brief Divide the vector by an double.
 *
 * \param[in] d	double to divide
 * \return the result of the division.
 */

inline V3D
V3D::operator/=(const double d)
{
  v[0] /= d;
  v[1] /= d;
  v[2] /= d;
  v[3] = 1;

  return *this;
}

/* -- V3D::operator* ------------------------------------------------- */

/** \brief Multiply the vector by a double.
 *
 * \param[in] d	double to multiply
 * \return the result of the multiplication.
 */

inline V3D
V3D::operator*(const double d) const
{
  V3D outV;

  outV.v[0] = v[0]*d;
  outV.v[1] = v[1]*d;
  outV.v[2] = v[2]*d;
  outV.v[3] = 1;
  return outV;
}


/* -- V3D::operator*= ------------------------------------------------ */

/** \brief Multiply the vector by a double.
 *
 * \param[in] d	double to multiply
 * \return the result of the multiplication.
 */

inline V3D
V3D::operator*=(const double d)
{
  v[0] *= d;
  v[1] *= d;
  v[2] *= d;
  v[3] = 1;

  return *this;
}


/* -- V3D::operator* ------------------------------------------------- */

/** \brief dot product
 *
 * \param[in] inV	Vector to multiply
 * \return the result of the dot product.
 */

inline double
V3D::operator*(const V3D& inV) const
{
  double outD;

  v3dDotIn(&outD, this, &inV);

  return outD;
}

/* -- V3D::operator^ ------------------------------------------------- */

/** \brief Vector product
 *
 * \param[in] inV	Vector to multiply
 * \return the result of the cross product.
 */

inline V3D
V3D::operator^(const V3D& inV) const
{
  V3D outV;

  v3dProdIn(&outV, this, &inV);

  return outV;
}

/* -- V3D::operator== ------------------------------------------------- */

/** \brief Vector comparison
 *
 * \param[in] inV	Vector to compare
 * \return the result of the comparison.
 */

inline bool 
V3D::operator==(const V3D& inV) const
{
	return (
		(inV.v[0] == v[0]) &&
		(inV.v[1] == v[1]) &&
		(inV.v[2] == v[2]) &&
		(inV.v[3] == v[3]));
};

/* --- V3D::operator[] ----------------------------------------------- */

/** \brief Vector element accessor.
 *
 * \param[in] i		Element number.
 * \return A reference on the element value.
 */

inline double &
V3D::operator[](unsigned int i)
{
  return v[i];
}

inline double
V3D::operator[](unsigned int i) const
{
    return v[i];
}

/** \copydoc V3D::operator[](int i) */
inline double &
V3D::operator[](V3DVectorIndex i)
{
  return v[i];
}


/* --- V3D::operator() ----------------------------------------------- */

/** \brief Vector element accessor.
 *
 * \param(in) i		Element number.
 * \return A reference on the element value.
 */

inline double &
V3D::operator()(unsigned int i)
{
  return v[i];
}

inline double
V3D::operator()(unsigned int i) const
{
    return v[i];
}

/* --- V3D::operator* ------------------------------------------------ */

/** \brief Apply a T3D to a V3D.
 *
 * \param[in] t3d	T3D to apply on vector.
 * \param[in] b		Vector to transform.
 * \return A new transformed vector.
 */


inline V3D
operator*(T3D &t3d, const V3D &b)
{
  V3D r(b);
  return *v3dApplyT3D(&t3d, &r);
}


/* --- V3D::norm ----------------------------------------------------- */

/** \brief Compute euclidian norm of a V3D
 *
 * \param[in] v3d	Input vector
 *
 * \return the euclidian norm of the vector.
 */

inline double
V3D::norm() const
{
  return v3dNorm(this);
}


/* --- V3D::normalize ------------------------------------------------ */

/** \brief Normalize a V3D
 *
 * \param[inout] v3d	Input vector.
 *
 * \return the normalized vector.
 */

inline V3D &
V3D::normalize()
{
  v3dNormalize(this);
  return *this;
}

#endif /* H_V3D_INLINE */
